@@ -12,13 +12,12 @@
 -include("../include/yaws_api.hrl").
 -include("yaws_debug.hrl").
 
-
-
 -include_lib("kernel/include/file.hrl").
 -export([start/0, stop/0, hup/1, restart/0, modules/0, load/0]).
 -export([start_embedded/1, start_embedded/2, start_embedded/3,
          start_embedded/4,
-         add_server/2]). 
+	 add_server/2,
+	 create_gconf/2, create_sconf/2]).
 -export([new_ssl/0,
          ssl_keyfile/1, ssl_keyfile/2,
          ssl_certfile/1, ssl_certfile/2,
@@ -39,7 +38,8 @@
          strip_spaces/1, strip_spaces/2,
          month/1, mk2/1, home/0,
          arg_rewrite/1, to_lowerchar/1, to_lower/1, funreverse/2, is_prefix/2,
-         split_sep/2, accepts_gzip/2, upto_char/2, deepmap/2,
+         split_sep/2, join_sep/2,
+         accepts_gzip/2, upto_char/2, deepmap/2,
          ticker/2, ticker/3]).
 
 -export([outh_set_status_code/1,
@@ -100,7 +100,7 @@
 -export([sconf_to_srvstr/1, 
          redirect_host/2, redirect_port/1,
          redirect_scheme_port/1, redirect_scheme/1,
-         tmpdir/0, split_at/2,
+         tmpdir/0, tmpdir/1, split_at/2,
          id_dir/1, ctl_file/1]).
          
          
@@ -137,14 +137,27 @@ start_embedded(DocRoot, SL, GL, Id) when is_list(DocRoot),is_list(SL),is_list(GL
     ok = application:set_env(yaws, embedded, true),
     ok = application:set_env(yaws, id, Id),
     application:start(yaws),
-    GC = setup_gconf(GL, yaws_config:make_default_gconf(false, Id)),
-    SC = setup_sconf(DocRoot, #sconf{}, SL),
+    GC = create_gconf(GL, Id),
+    SC = create_sconf(DocRoot, SL),
     yaws_config:add_yaws_soap_srv(GC),
-    yaws_api:setconf(GC, [SC]).
+    yaws_api:setconf(GC, [[SC]]).
 
 add_server(DocRoot, SL) when is_list(DocRoot),is_list(SL) ->
-    SC = setup_sconf(DocRoot, #sconf{}, SL),
-    yaws_config:add_sconf(SC).
+    SC  = create_sconf(DocRoot, SL),
+    %% Change #auth in authdirs to {Dir, #auth} if needed
+    Fun = fun
+	      (A = #auth{dir = [Dir]}, Acc) -> [{Dir, A}| Acc];
+	      (A, Acc)                      -> [A| Acc]
+	  end,
+    Authdirs = lists:foldr(Fun, [], SC#sconf.authdirs),
+    yaws_config:add_sconf(SC#sconf{authdirs = Authdirs}).
+
+create_gconf(GL, Id) when is_list(GL) ->
+    setup_gconf(GL, yaws_config:make_default_gconf(false, Id)).
+
+create_sconf(DocRoot, SL) when is_list(DocRoot), is_list(SL) ->
+    setup_sconf(DocRoot, #sconf{}, SL).
+
 
 %%% Access functions for the SSL record.
 new_ssl()             -> #ssl{}.
@@ -196,8 +209,8 @@ setup_gconf(GL, GC) ->
                                 GC#gconf.log_wrap_size),
            cache_refresh_secs = lkup(cache_refresh_secs, GL, 
                                      GC#gconf.cache_refresh_secs),
-            mnesia_dir = lkup(mnesia_dir, GL,
-			      GC#gconf.mnesia_dir),
+           mnesia_dir = lkup(mnesia_dir, GL,
+                             GC#gconf.mnesia_dir),
            include_dir = lkup(include_dir, GL, 
                               GC#gconf.include_dir),
            phpexe = lkup(phpexe, GL, 
@@ -218,6 +231,8 @@ set_gc_flags([{auth_log, Bool}|T], Flags) ->
     set_gc_flags(T, flag(Flags, ?GC_AUTH_LOG, Bool));
 set_gc_flags([{copy_errlog, Bool}|T], Flags) -> 
     set_gc_flags(T, flag(Flags, ?GC_COPY_ERRLOG, Bool));
+set_gc_flags([{copy_error_log, Bool}|T], Flags) ->
+    set_gc_flags(T, flag(Flags, ?GC_COPY_ERRLOG, Bool));
 set_gc_flags([{backwards_compat_parse, Bool}|T], Flags) -> 
     set_gc_flags(T, flag(Flags, ?GC_BACKWARDS_COMPAT_PARSE, Bool));
 set_gc_flags([{log_resolve_hostname, Bool}|T], Flags) -> 
@@ -254,14 +269,15 @@ setup_sconf(DocRoot, D, SL) ->
                              D#sconf.servername),
            ets = lkup(ets, SL, 
                       D#sconf.ets),
-           ssl = lkup(ssl, SL, 
-                      D#sconf.ssl),
+           ssl = setup_sconf_ssl(SL, D#sconf.ssl),
            authdirs = lkup(authdirs, SL, 
                            D#sconf.authdirs),
            partial_post_size = lkup(partial_post_size, SL, 
                                     D#sconf.partial_post_size),
            appmods = lkup(appmods, SL, 
                           D#sconf.appmods),
+           errormod_401 = lkup(errormod_401, SL,
+                               D#sconf.errormod_401),
            errormod_404 = lkup(errormod_404, SL, 
                                D#sconf.errormod_404),
            errormod_crash = lkup(errormod_crash, SL, 
@@ -278,7 +294,38 @@ setup_sconf(DocRoot, D, SL) ->
                            D#sconf.revproxy),
            soptions = lkup(soptions, SL,
                            D#sconf.soptions),
-	   stats = lkup(soptions, SL, D#sconf.stats)}.
+           stats = lkup(stats, SL, D#sconf.stats),
+           fcgi_app_server_host = lkup(fcgi_app_server_host, SL, 
+                                       D#sconf.fcgi_app_server_host),
+           fcgi_app_server_port = lkup(fcgi_app_server_port, SL, 
+                                       D#sconf.fcgi_app_server_port)}.
+
+setup_sconf_ssl(SL, DefaultSSL) ->
+    case lkup(ssl, SL, undefined) of
+	undefined ->
+	    DefaultSSL;
+	SSL when is_record(SSL, ssl) ->
+	    SSL;
+	SSLProps when is_list(SSLProps) ->
+	    SSL1 = #ssl{
+	      keyfile = proplists:get_value(keyfile, SSLProps),
+	      certfile = proplists:get_value(certfile, SSLProps),
+	      password = proplists:get_value(password, SSLProps),
+	      cacertfile = proplists:get_value(cacertfile, SSLProps),
+	      ciphers = proplists:get_value(ciphers, SSLProps),
+	      cachetimeout = proplists:get_value(cachetimeout, SSLProps)
+	     },
+	    %% Prevent overriding the ssl record's default values!
+	    SSL2 =
+		case proplists:get_value(verify, SSLProps) of
+		    undefined -> SSL1;
+		    Verify -> SSL1#ssl{verify=Verify}
+		end,
+	    case proplists:get_value(depth, SSLProps) of
+		undefined -> SSL2;
+		Depth -> SSL2#ssl{depth=Depth}
+	    end
+    end.
 
 set_sc_flags([{access_log, Bool}|T], Flags) ->
     set_sc_flags(T, flag(Flags, ?SC_ACCESS_LOG, Bool));
@@ -296,7 +343,12 @@ set_sc_flags([{dir_all_zip, Bool}|T], Flags) ->
     set_sc_flags(T, flag(Flags, ?SC_DIR_ALL_ZIP, Bool));
 set_sc_flags([{dav, Bool}|T], Flags) ->
     set_sc_flags(T, flag(Flags, ?SC_DAV, Bool));
-set_sc_flags([_|T], Flags) ->
+set_sc_flags([{fcgi_trace_protocol, Bool}|T], Flags) ->
+    set_sc_flags(T, flag(Flags, ?SC_FCGI_TRACE_PROTOCOL, Bool));
+set_sc_flags([{forward_proxy, Bool}|T], Flags) ->
+    set_sc_flags(T, flag(Flags, ?SC_FORWARD_PROXY, Bool));
+set_sc_flags([_Unknown|T], Flags) ->
+    error_logger:format("Unknown and unhandled flag ~p~n", [_Unknown]),
     set_sc_flags(T, Flags);
 set_sc_flags([], Flags) ->
     Flags.
@@ -704,7 +756,7 @@ printversion() ->
     io:format("Yaws ~s~n", [yaws_generated:version()]),
     init:stop().
 
-%% our default arg rewriteer does's of cource nothing
+%% our default arg rewriter does of course nothing
 arg_rewrite(A) ->
     A.
 
@@ -756,7 +808,7 @@ is_prefix(_,_) ->
     false.
 
 
-%% Split a string of words seperated by Sep into a list of words and
+%% Split a string of words separated by Sep into a list of words and
 %% strip off white space.
 %%
 %% HTML semantics are used, such that empty words are omitted.
@@ -788,6 +840,15 @@ split_sep([C|Tail], Sep, AccW, AccL) ->
     split_sep(Tail, Sep, [C|AccW], AccL).
 
 
+%% Join strings with separator. Same as string:join in later
+%% versions of Erlang. Separator is expected to be a list.
+
+join_sep([], Sep) when is_list(Sep) ->
+    [];
+join_sep([H|T], Sep) ->
+    H ++ lists:append([Sep ++ X || X <- T]).
+
+
 %% header parsing
 
 parse_qval(S) ->
@@ -1603,7 +1664,7 @@ split_header([C|S], A) ->
 
 
 erase_header(connection) ->
-    put(outh, (get(outh))#outh{connection = undefined});
+    put(outh, (get(outh))#outh{connection = undefined, doclose = false});
 erase_header(cache_control) ->
     put(outh, (get(outh))#outh{cache_control = undefined});
 erase_header(set_cookie) ->
@@ -1711,7 +1772,6 @@ do_recv(Sock, Num, nossl) ->
 do_recv(Sock, Num, ssl) ->
     ssl:recv(Sock, Num, ?READ_TIMEOUT).
 
-
 cli_recv(S, Num, SslBool) ->
     Res = do_recv(S, Num, SslBool),
     cli_recv_trace((get(gc))#gconf.trace, Res),
@@ -1820,7 +1880,6 @@ setopts(Sock, Opts, ssl) ->
     ok = ssl:setopts(Sock, Opts).
 
 do_http_get_headers(CliSock, SSL) ->
-    setopts(CliSock, [{packet, http}], SSL),
     case http_recv_request(CliSock,SSL) of
         bad_request ->
             {#http_request{method=bad_request, version={0,9}},
@@ -1834,6 +1893,7 @@ do_http_get_headers(CliSock, SSL) ->
 
 
 http_recv_request(CliSock, SSL) ->
+    setopts(CliSock, [{packet, http}], SSL),
     case do_recv(CliSock, 0,  SSL) of
         {ok, R} when is_record(R, http_request) ->
             R;
@@ -1849,13 +1909,12 @@ http_recv_request(CliSock, SSL) ->
             closed;
         {error, timeout} -> closed;
         _Other ->
-            ?Debug("Got ~p~n", [_Other]),
+            error_logger:format("Unhandled reply fr. do_recv() ~p~n", [_Other]),
             exit(normal)
     end.
 
-
-
 http_collect_headers(CliSock, Req, H, SSL, Count) when Count < 1000 ->
+    setopts(CliSock, [{packet, httph}], SSL),
     Recv = do_recv(CliSock, 0, SSL),
     case Recv of
         {ok, {http_header,  _Num, 'Host', _, Host}} ->
@@ -1966,24 +2025,27 @@ parse_auth(_) ->
     undefined.
 
 
-
 decode_base64([]) ->
     [];
-decode_base64([Sextet1,Sextet2,$=,$=|Rest]) ->
+decode_base64(Auth64) ->
+    decode_base64(Auth64, []).
+decode_base64([], Acc) ->
+    lists:reverse(Acc);
+decode_base64([Sextet1,Sextet2,$=,$=|Rest], Acc) ->
     Bits2x6=
         (d(Sextet1) bsl 18) bor
         (d(Sextet2) bsl 12),
     Octet1=Bits2x6 bsr 16,
-    [Octet1|decode_base64(Rest)];
-decode_base64([Sextet1,Sextet2,Sextet3,$=|Rest]) ->
+    decode_base64(Rest, [Octet1|Acc]);
+decode_base64([Sextet1,Sextet2,Sextet3,$=|Rest], Acc) ->
     Bits3x6=
         (d(Sextet1) bsl 18) bor
         (d(Sextet2) bsl 12) bor
         (d(Sextet3) bsl 6),
     Octet1=Bits3x6 bsr 16,
     Octet2=(Bits3x6 bsr 8) band 16#ff,
-    [Octet1,Octet2|decode_base64(Rest)];
-decode_base64([Sextet1,Sextet2,Sextet3,Sextet4|Rest]) ->
+    decode_base64(Rest, [Octet2,Octet1|Acc]);
+decode_base64([Sextet1,Sextet2,Sextet3,Sextet4|Rest], Acc) ->
     Bits4x6=
         (d(Sextet1) bsl 18) bor
         (d(Sextet2) bsl 12) bor
@@ -1992,8 +2054,8 @@ decode_base64([Sextet1,Sextet2,Sextet3,Sextet4|Rest]) ->
     Octet1=Bits4x6 bsr 16,
     Octet2=(Bits4x6 bsr 8) band 16#ff,
     Octet3=Bits4x6 band 16#ff,
-    [Octet1,Octet2,Octet3|decode_base64(Rest)];
-decode_base64(_CatchAll) ->
+    decode_base64(Rest, [Octet3,Octet2,Octet1|Acc]);
+decode_base64(_CatchAll, _Acc) ->
     {error, bad_base64}.
 
 d(X) when X >= $A, X =<$Z ->
@@ -2127,6 +2189,8 @@ redirect_scheme_port(SC) ->
     {Scheme, PortPart}.
 
 tmpdir() ->
+    tmpdir(filename:join([home(), ".yaws"])).
+tmpdir(DefaultTmpDir) ->
     case os:type() of
         {win32,_} ->
             case os:getenv("TEMP") of
@@ -2150,10 +2214,10 @@ tmpdir() ->
                     PathTEMP
             end;
         _ ->
-	    filename:join([home(), ".yaws"])
+	    DefaultTmpDir
     end.
 
-%% This feature is useable together with 
+%% This feature is usable together with 
 %% privbind and authbind on linux
 
 home() ->
@@ -2196,6 +2260,8 @@ nonl([10|T]) ->
     nonl(T);
 nonl([13|T]) ->
     nonl(T);
+nonl([32|T]) ->
+    nonl(T);
 nonl([H|T]) ->
     [H|nonl(T)];
 nonl([]) ->
