@@ -8,7 +8,7 @@
 -module(yaws_api).
 -author('klacke@hyber.org').
 
-%% -compile(expot_all).
+%% -compile(export_all).
 
 
 -include("../include/yaws.hrl").
@@ -34,6 +34,10 @@
 -export([get_line/1, mime_type/1]).
 -export([stream_chunk_deliver/2, stream_chunk_deliver_blocking/2,
          stream_chunk_end/1]).
+-export([stream_process_deliver/2, stream_process_deliver_chunk/2,
+         stream_process_deliver_final_chunk/2, stream_process_end/2]).
+-export([websocket_send/2, websocket_receive/1,
+         websocket_unframe_data/1, websocket_setopts/2]).
 -export([new_cookie_session/1, new_cookie_session/2, new_cookie_session/3, 
          cookieval_to_opaque/1, request_url/1,
          print_cookie_sessions/0,
@@ -51,6 +55,9 @@
 
 -export([call_cgi/2, call_cgi/3]).
 
+-export([call_fcgi_responder/1, call_fcgi_responder/2,
+         call_fcgi_authorizer/1, call_fcgi_authorizer/2]).
+
 -export([ehtml_expand/1, ehtml_expander/1, ehtml_apply/2,
          ehtml_expander_test/0]).
 
@@ -505,7 +512,17 @@ code_to_phrase(501) -> "Not Implemented";
 code_to_phrase(502) -> "Bad Gateway";
 code_to_phrase(503) -> "Service Unavailable";
 code_to_phrase(504) -> "Gateway Timeout";
-code_to_phrase(505) -> "HTTP Version Not Supported".
+code_to_phrase(505) -> "HTTP Version Not Supported";
+
+%% Below are some non-HTTP status codes from other protocol standards that
+%% we've seen used with HTTP in the wild, so we include them here. HTTP 1.1
+%% section 6.1.1 allows for this sort of extensibility, but we recommend
+%% sticking with the HTTP status codes above for maximal portability and
+%% interoperability.
+%%
+code_to_phrase(451) -> "Requested Action Aborted";   % from FTP (RFC 959)
+code_to_phrase(452) -> "Insufficient Storage Space"; % from FTP (RFC 959)
+code_to_phrase(453) -> "Not Enough Bandwidth".       % from RTSP (RFC 2326)
 
 
 
@@ -831,6 +848,68 @@ stream_chunk_deliver_blocking(YawsPid, Data) ->
 stream_chunk_end(YawsPid) ->
     YawsPid ! endofstreamcontent.
 
+stream_process_deliver(Sock={sslsocket,_,_}, IoList) ->
+    ssl:send(Sock, IoList);
+stream_process_deliver(Sock, IoList) ->
+    gen_tcp:send(Sock, IoList).
+
+stream_process_deliver_chunk(Sock, IoList) ->
+    Chunk = case erlang:iolist_size(IoList) of
+                0 ->
+                    stream_process_deliver_final_chunk(Sock, IoList);
+                S ->
+                    [yaws:integer_to_hex(S), "\r\n", IoList, "\r\n"]
+            end,
+    stream_process_deliver(Sock, Chunk).
+
+stream_process_deliver_final_chunk(Sock, IoList) ->
+    Chunk = case erlang:iolist_size(IoList) of
+                0 ->
+                    <<"0\r\n\r\n">>;
+                S ->
+                    [yaws:integer_to_hex(S), "\r\n", IoList, "\r\n0\r\n\r\n"]
+            end,
+    stream_process_deliver(Sock, Chunk).
+
+stream_process_end(Sock={sslsocket,_,_}, YawsPid) ->
+    ssl:controlling_process(Sock, YawsPid),
+    YawsPid ! endofstreamcontent;
+stream_process_end(Sock, YawsPid) ->
+    gen_tcp:controlling_process(Sock, YawsPid),
+    YawsPid ! endofstreamcontent.
+
+
+websocket_send(Socket, IoList) ->
+    DataFrame = [0, IoList, 255],
+    case Socket of
+	{sslsocket,_,_} ->
+	    ssl:send(Socket, DataFrame);
+	_ ->
+	    gen_tcp:send(Socket, DataFrame)
+    end.
+
+websocket_receive(Socket) ->
+    R = case Socket of
+	    {sslsocket,_,_} ->
+		ssl:recv(Socket, 0);
+	    _ ->
+		gen_tcp:recv(Socket, 0)
+	end,
+    case R of
+	{ok, DataFrames} ->
+	    ReceivedMsgs = yaws_websockets:unframe_all(DataFrames, []),
+	    {ok, ReceivedMsgs};
+	_ -> R
+    end.
+
+websocket_unframe_data(DataFrameBin) ->
+    {ok, Msg, <<>>} = yaws_websockets:unframe_one(DataFrameBin),
+    Msg.
+
+websocket_setopts({sslsocket,_,_}=Socket, Opts) ->
+    ssl:setopts(Socket, Opts);
+websocket_setopts(Socket, Opts) ->
+    inet:setopts(Socket, Opts).
 
 
 %% Return new cookie string
@@ -1476,6 +1555,58 @@ call_cgi(Arg, Scriptfilename) ->
 call_cgi(Arg, Exefilename, Scriptfilename) -> 
     yaws_cgi:call_cgi(Arg, Exefilename, Scriptfilename).
 
+%% call_fci_responder issues a responder role call to the FastCGI 
+%% application server. It returns the same return value as out/1.
+%%
+%% call_fci_authorizer issues a authorizer role call to the FastCGI 
+%% application server. It returns:
+%%
+%% {denied, Out} : Access is denied. Out is the same return value as 
+%% out/1.
+%%
+%% {allowed, Variables} : Access is allowed. Variables is a list of
+%% environment variables returned by the authorization server using
+%% Variable-XXX: YYY headers.
+%%
+%% Note: the FastCGI filter role is not yet supported. 
+%%
+%% The following information is taken from the server configuration:
+%% - The hostname (or address) and port number of the application server.
+%% - Extra CGI variables.
+%% - Trace FastCGI protocol messages?
+%% - Log application server error messages?
+%%
+%% The caller can optionally provide an Options argument which supports
+%% the following options. These override the defaults taken from the
+%% server config.
+%%
+%% {app_server_host, string() | ip_address()} : The hostname or IP address
+%% of the application server.
+%%
+%% {app_server_port, int()} : The TCP port number of the application server.
+%%
+%% {path_info, string()} : Override the patinfo string from Arg.
+%%
+%% {extra_env, [{string(), string()}]} : Extra environment variables to be
+%% passed to the application server, as a list of name-value pairs.
+%%
+%% trace_protocol : Trace FastCGI protocol messages.
+%%
+%% log_app_error : Log application errors (output to stderr and non-zero
+%% exit value).
+%%
+call_fcgi_responder(Arg) ->
+    yaws_cgi:call_fcgi_responder(Arg).
+
+call_fcgi_responder(Arg, Options) ->
+    yaws_cgi:call_fcgi_responder(Arg, Options).
+
+call_fcgi_authorizer(Arg) ->
+    yaws_cgi:call_fcgi_authorizer(Arg).
+
+call_fcgi_authorizer(Arg, Options) ->
+    yaws_cgi:call_fcgi_authorizer(Arg, Options).
+
 %%
 
 deepmember(_C,[]) ->
@@ -1831,7 +1962,12 @@ redirect_self(A) ->
                 {Port2, [$:|integer_to_list(Port2)]}
         end,
     H = A#arg.headers,
-    Host = yaws:redirect_host(get(sc), H#headers.host),
+    Host0 = yaws:redirect_host(get(sc), H#headers.host),
+    %% redirect host contains the port number - for mysterious reasons
+    Host = case string:tokens(Host0, ":") of
+               [H0, _] -> H0;
+               [H1] -> H1
+           end,
     {Scheme, SchemeStr} = 
         case {SC#sconf.ssl,SC#sconf.rmethod} of
             {_, Method} when is_list(Method) ->
