@@ -1,6 +1,6 @@
-%%----------------------------------------------------------------------
+%%%----------------------------------------------------------------------
 %%% File    : yaws_config.erl
-%% Author  : Claes Wikstrom <klacke@bluetail.com>
+%%% Author  : Claes Wikstrom <klacke@bluetail.com>
 %%% Purpose : 
 %%% Created : 16 Jan 2002 by Claes Wikstrom <klacke@bluetail.com>
 %%%----------------------------------------------------------------------
@@ -17,7 +17,6 @@
 -include_lib("kernel/include/file.hrl").
 
 -export([load/1,
-         toks/1,
          make_default_gconf/2, make_default_sconf/0,
          add_sconf/1,
          add_yaws_auth/1,
@@ -83,7 +82,7 @@ load(E) ->
 add_yaws_soap_srv(GC) when GC#gconf.enable_soap == true ->
     SoapStarted = (whereis(yaws_soap_srv) /= undefined),
     if (SoapStarted == false) ->
-            Spec = {yaws_soap_srv, {yaws_soap_srv, start_link, []},
+            Spec = {yaws_soap_srv, {yaws_soap_srv, start_link, [GC#gconf.soap_srv_mods] },
                     permanent, 5000, worker, [yaws_soap_srv]},
             spawn(fun() -> supervisor:start_child(yaws_sup, Spec) end);
        true ->
@@ -94,123 +93,101 @@ add_yaws_soap_srv(_GC) ->
 
 
 add_yaws_auth(SCs) ->
-    lists:map(
-      fun(SC) ->
-              SC#sconf{authdirs = setup_auth(SC)}
-      end, SCs).
+    [SC#sconf{authdirs = setup_auth(SC)} || SC <- SCs].
 
 
 %% We search and setup www authenticate for each directory
 %% specified as an auth directory or containing a .yaws_auth file. 
 %% These are merged with server conf.
-
-setup_auth(SC) ->
-    Auth_dirs0 = get_yaws_auth_dirs(SC#sconf.docroot),
-
-    %% create new auth records
-    Auth_dirs1 = [#auth{dir = [X]} || X <- Auth_dirs0],
-    Auth_dirs2 = Auth_dirs1 ++ SC#sconf.authdirs,
-    
-    %% load the .yaws_auth files and parse them
-    Auth_dirs3 = load_yaws_auth_file(SC, Auth_dirs2, []),
-
-    start_pam(Auth_dirs3),
-    Auth_dirs3.
+setup_auth(#sconf{docroot = Docroot, authdirs = Authdirs}) ->
+    Authdirs1 = load_yaws_auth_from_docroot(Docroot),
+    Authdirs2 = load_yaws_auth_from_authdirs(Authdirs, Docroot, Authdirs1),
+    Authdirs3 = ensure_auth_headers(Authdirs2),
+    Authdirs4 = [{Dir, A} || A = #auth{dir = [Dir]} <- Authdirs3], % A->{Dir, A}
+    start_pam(Authdirs4),
+    Authdirs4.
 
     
-
-%% Call get_yaws_auth_dirs/3 with default values and then
-%% strip leading docroot from dir
-get_yaws_auth_dirs(undefined) ->
+load_yaws_auth_from_docroot(undefined) ->
     [];
-get_yaws_auth_dirs(Docroot) ->
-    case file:list_dir(Docroot) of
-        {ok, FileList} ->
-            Auth_dirs = get_yaws_auth_dirs(Docroot ++ "/", FileList, []),
-            Len = string:len(Docroot),
-            [string:sub_string(X, Len+1) || X <- Auth_dirs];
-        _ ->
-            []
+load_yaws_auth_from_docroot(Docroot) ->
+    Fun = fun (Path, Acc) ->
+		  %% Strip Docroot and then filename
+		  SP  = string:sub_string(Path, length(Docroot)+1),
+		  Dir = filename:dirname(SP),
+		  case load_yaws_auth_file(Path, #auth{dir = [Dir]}) of
+		      Auth when is_record(Auth, auth)  -> 
+                          [Auth| Acc];
+		      _Other -> 
+                          Acc
+		  end
+	  end,
+    filelib:fold_files(Docroot, "^.yaws_auth$", true, Fun, []).
+
+
+load_yaws_auth_from_authdirs([], _, Acc) ->
+    lists:reverse(Acc);
+load_yaws_auth_from_authdirs([Auth = #auth{dir = [D]}| Rest], Docroot, Acc) ->
+    Dir     = case D of
+		  "/" ++ Rest -> Rest;	% strip leading slash
+		  _           -> D
+	      end,
+    Path    = filename:join([Docroot, Dir, ".yaws_auth"]),
+    NewAuth = case catch load_yaws_auth_file(Path, Auth) of
+		  {ok, A} -> A;
+		  _       -> Auth
+	      end,
+    load_yaws_auth_from_authdirs(Rest, Docroot, [NewAuth| Acc]);
+load_yaws_auth_from_authdirs([{_Dir, Auth}| Rest], Docroot, Acc) ->
+    %% handle {Dir, A} by stripping it
+    load_yaws_auth_from_authdirs([Auth| Rest], Docroot, Acc);
+load_yaws_auth_from_authdirs([_| Rest], Docroot, Acc) ->
+    load_yaws_auth_from_authdirs(Rest, Docroot, Acc).
+    
+
+load_yaws_auth_file(Path, Auth) ->
+    case file:consult(Path) of
+	{ok, TermList} ->
+	    error_logger:info_msg("Reading .yaws_auth ~s~n", [Path]),
+	    parse_yaws_auth_file(TermList, Auth);
+	{error, enoent} ->
+	    {error, enoent};
+	Error ->
+	    error_logger:format("Bad .yaws_auth file ~s ~p~n", [Path, Error]),
+	    Error
     end.
     
 
-%% Bottom of recursion, we have searched all the entries in this directory
-get_yaws_auth_dirs(_Docroot, [], AuthDirs) ->
-    AuthDirs;
+ensure_auth_headers(Authdirs) ->
+    [add_auth_headers(Auth) || Auth <- Authdirs].
+
+add_auth_headers(Auth = #auth{headers = []}) ->
+    %% Headers needs to be set
+    Realm   = Auth#auth.realm,
+    Headers = yaws:make_www_authenticate_header({realm, Realm}),
+    Auth#auth{headers = Headers};
+add_auth_headers(Auth) ->
+    Auth.
 
-%% Recursivly search the docroot for any dir that contains .yaws_auth 
-%% and return it
-get_yaws_auth_dirs(Docroot, [File|T], AuthDirs0) ->
-    Path = string:concat(Docroot, File),
-    case filelib:is_dir(Path) of
-	true ->
-            case file:list_dir(Path) of
-                {ok, FileList} ->
-                    AuthDirs1 = get_yaws_auth_dirs(Path ++ "/", FileList, 
-                                                   AuthDirs0),
-                    get_yaws_auth_dirs(Docroot, T, AuthDirs1);
-                _ ->
-                    []
-            end;
-	false ->
-	    case File of
-		".yaws_auth" ->
-		    get_yaws_auth_dirs(Docroot, T, [Docroot|AuthDirs0]);
-		_ ->
-		    get_yaws_auth_dirs(Docroot, T, AuthDirs0)
-	    end
-    end.
 
 start_pam([]) ->
     ok;
-
+start_pam([{_Dir, #auth{pam = false}}|T]) ->
+    start_pam(T);
 start_pam([{_Dir, A}|T]) ->
-    PamStarted = whereis(yaws_pam) /= undefined,
-    if
-        A#auth.pam == false ->
-            ok;
-        PamStarted == false ->
+    case whereis(yaws_pam) of
+	undefined ->	% pam not started
             Spec = {yaws_pam, {yaws_pam, start_link, 
                                [yaws:to_list(A#auth.pam),undefined,undefined]},
                     permanent, 5000, worker, [yaws_pam]},
-            spawn(fun() ->
-                          supervisor:start_child(yaws_sup, Spec)
-                  end);
-        true ->
-            ok
-    end,
-    start_pam(T).
+	    spawn(fun() -> supervisor:start_child(yaws_sup, Spec) end);
+	_ ->
+	    start_pam(T)
+    end.
 
 
-
-load_yaws_auth_file(_SC, [], Acc) ->
-    Acc;
-
-%% Load the .yaws_auth file if it exists and parse it, 
-%% adding the result to the record
-load_yaws_auth_file(SC, [A|T], Acc) ->
-    [Dir] = A#auth.dir,
-    FN0=[SC#sconf.docroot, [$/|Dir], [$/|".yaws_auth"]],
-    FN1 = remove_multiple_slash(lists:flatten(FN0)),
-    A2 = case file:consult(FN1) of
-	     {ok, TermList} ->
-		 error_logger:info_msg("Reading .yaws_auth ~s~n",[FN1]),
-		 Auth = parse_yaws_auth_file(TermList, A),
-		 [Dir1] = Auth#auth.dir,
-		 {Dir1, Auth};
-	     {error, enoent} ->
-		 {Dir, A};
-	     _Err ->
-		 error_logger:format("Bad .yaws_auth file in dir ~p~n", 
-				     [Dir]),
-		 {Dir, A}
-	 end,
-    load_yaws_auth_file(SC, T, [A2|Acc]). 
-
-parse_yaws_auth_file([], Auth0) ->
-    Realm = Auth0#auth.realm,
-    Headers = Auth0#auth.headers ++ yaws:make_www_authenticate_header({realm, Realm}),
-    Auth0#auth{headers = Headers};
+parse_yaws_auth_file([], Auth) ->
+    Auth;
 
 parse_yaws_auth_file([{realm, Realm}|T], Auth0) ->
     parse_yaws_auth_file(T, Auth0#auth{realm = Realm});
@@ -242,17 +219,18 @@ parse_yaws_auth_file([{User, Password}|T], Auth0)
     parse_yaws_auth_file(T, Auth0#auth{users = Users}).
 
 
+%% Not used anymore
 %% Replace all "//" and "///" with "/"
 %% Might be a better way to do this
-remove_multiple_slash(L) ->
-    remove_multiple_slash(L, []).
+%% remove_multiple_slash(L) ->
+%%     remove_multiple_slash(L, []).
 
-remove_multiple_slash([], Acc)->
-    lists:reverse(Acc);
-remove_multiple_slash("//" ++ T, Acc) ->
-    remove_multiple_slash([$/|T], Acc);
-remove_multiple_slash([H|T], Acc) ->    
-    remove_multiple_slash(T, [H|Acc]).
+%% remove_multiple_slash([], Acc)->
+%%     lists:reverse(Acc);
+%% remove_multiple_slash("//" ++ T, Acc) ->
+%%     remove_multiple_slash([$/|T], Acc);
+%% remove_multiple_slash([H|T], Acc) ->
+%%     remove_multiple_slash(T, [H|Acc]).
 
 %% This is the function that arranges sconfs into
 %% different server groups
@@ -279,14 +257,22 @@ validate_groups([H|T]) ->
     end.
 
 validate_group(List) ->
-    %% first, max one ssl per group
-    case lists:filter(fun(SC) -> 
-                              SC#sconf.ssl /= undefined 
-                      end, List) of
-        L when length(L) > 1 ->
-            throw({error, ?F("Max one ssl server per IP: ~p", 
-                             [(hd(L))#sconf.servername])});
-        _ ->
+    %% all ssl servers with the same IP must share the same ssl  configuration
+    %% check if one ssl server in the group
+    case lists:any(fun(SC) -> SC#sconf.ssl  /= undefined end,List) of 
+	true ->
+            [SC0|SCs] = List,
+            %% check if all servers in the group have the same ssl configuration
+            case lists:filter(
+                   fun(SC) -> SC#sconf.ssl /= SC0#sconf.ssl end,SCs) of
+                L when length(L) > 1 ->
+                    throw({error, ?F("SSL server per IP must share the "
+                                     "same certificate : ~p", 
+                                     [(hd(L))#sconf.servername])});
+                _ ->
+                    ok
+            end;
+	_ -> 
             ok
     end,
     %% second all servernames in a group must be unique
@@ -411,6 +397,23 @@ del_tail2([H|T], Acc) ->
 
 
 
+string_to_host_and_port(String) ->
+    case string:rchr(String, $:) of
+        0 ->
+            {error, "missing colon (:) character."};
+        ColonPos ->
+            Host = string:left(String, ColonPos - 1),
+            PortStr = string:sub_string(String, ColonPos + 1),
+            case (catch list_to_integer(PortStr)) of
+                Port when is_integer(Port) and (Port >= 0) and (Port =< 65535) ->
+                    {ok, Host, Port};
+                _ ->
+                    {error, ?F("~p is not a valid port number.", [PortStr])}
+            end
+    end.
+
+
+
 %% two states, global, server
 fload(FD, globals, GC, _C, Cs, _Lno, eof) ->
     file:close(FD),
@@ -423,7 +426,7 @@ fload(FD, _,  _GC, _C, _Cs, Lno, eof) ->
 
 fload(FD, globals, GC, C, Cs, Lno, Chars) -> 
     Next = io:get_line(FD, ''),
-    case toks(Chars) of
+    case toks(Lno, Chars) of
         [] ->
             fload(FD, globals, GC, C, Cs, Lno+1, Next);
 
@@ -544,7 +547,7 @@ fload(FD, globals, GC, C, Cs, Lno, Chars) ->
                           C, Cs, Lno+1, Next);
                 false ->
                     fload(FD, globals, GC, C, Cs, Lno+1, Next)
-	    
+
             end;
 
         ["runmod", '=', Mod0] ->
@@ -559,6 +562,26 @@ fload(FD, globals, GC, C, Cs, Lno, Chars) ->
                true ->
                     fload(FD, globals, GC#gconf{enable_soap = false},
                           C, Cs, Lno+1, Next)
+            end;		
+
+        ["soap_srv_mods", '=' | SoapSrvMods] ->
+            case parse_soap_srv_mods(SoapSrvMods, []) of
+                {ok, L} ->
+                    fload(FD, globals, GC#gconf{soap_srv_mods = L}, 
+						  C, Cs, Lno+1, Next);
+                {error, Str} ->
+                    {error, ?F("~s at line ~w", [Str, Lno])}
+            end;
+
+        ["max_connections", '=', Int] ->
+            case (catch list_to_integer(Int)) of
+                I when is_integer(I) ->
+                    fload(FD, globals, GC#gconf{max_connections = I},
+                          C, Cs, Lno+1, Next);
+                _ when Int == "nolimit" ->
+                    fload(FD, globals, GC, C, Cs, Lno+1, Next);
+                _ ->
+                    {error, ?F("Expect integer at line ~w", [Lno])}
             end;
 
         ["log_wrap_size", '=', Int] ->
@@ -701,7 +724,7 @@ fload(FD, globals, GC, C, Cs, Lno, Chars) ->
                     {error, ?F("Expect true|false at line ~w", [Lno])}
             end;
         ["id", '=', String] when GC#gconf.id == undefined;  
-                                 GC#gconf.id == default ->
+                                 GC#gconf.id == "default" ->
             fload(FD, globals, GC#gconf{id=String},C, Cs, Lno+1, Next);
         ["id", '=', String]  ->
             error_logger:format("Ignoring 'id = ~p' setting at line ~p~n", 
@@ -744,13 +767,15 @@ fload(FD, globals, GC, C, Cs, Lno, Chars) ->
                   Cs, Lno+1, Next);
 
         [H|_] ->
-            {error, ?F("Unexpected tokens ~p at line ~w", [H, Lno])}
+            {error, ?F("Unexpected tokens ~p at line ~w", [H, Lno])};
+        Err ->
+            Err
     end;
 
 
 fload(FD, server, GC, C, Cs, Lno, Chars) ->
     Next = io:get_line(FD, ''),
-    case toks(Chars) of
+    case toks(Lno, Chars) of
         [] ->
             fload(FD, server, GC, C, Cs, Lno+1, Next);
 
@@ -830,13 +855,16 @@ fload(FD, server, GC, C, Cs, Lno, Chars) ->
         ["listen_backlog", '=', Val] ->
             case (catch list_to_integer(Val)) of
                 B when is_integer(B) ->
-                    C2 = case proplists:get_value(listen_opts, C#sconf.soptions) of
+                    C2 = case proplists:get_value(listen_opts, 
+                                                  C#sconf.soptions) of
                              undefined ->
-                                 C#sconf{soptions = [{listen_opts, [{backlog, B}]} |
-                                                     C#sconf.soptions]};
+                                 C#sconf{soptions = 
+                                         [{listen_opts, [{backlog, B}]} |
+                                          C#sconf.soptions]};
                              Opts ->
-                                 C#sconf{soptions = [{listen_opts, [{backlog, B} | Opts]} |
-                                                     C#sconf.soptions]}
+                                 C#sconf{soptions = 
+                                         [{listen_opts, [{backlog, B} | Opts]} |
+                                          C#sconf.soptions]}
                          end,
                     fload(FD, server, GC, C2, Cs, Lno+1, Next);
                 _ ->
@@ -858,12 +886,12 @@ fload(FD, server, GC, C, Cs, Lno, Chars) ->
                     fload(FD, server, GC, C2, Cs, Lno+1, Next);
                 _ ->
                     {error, ?F("Expect directory at line ~w (docroot: ~s)", 
-			       [Lno, hd(RootDirs)])}
+                               [Lno, hd(RootDirs)])}
             end;
 
         ["partial_post_size",'=',Size] ->
             case Size of
-                nolimit ->
+                "nolimit" ->
                     C2 = C#sconf{partial_post_size = nolimit},
                     fload(FD, server, GC, C2, Cs, Lno+1, Next);
                 Val ->
@@ -907,9 +935,9 @@ fload(FD, server, GC, C, Cs, Lno, Chars) ->
             C2 = C#sconf{errormod_crash = list_to_atom(Module)},
             fload(FD, server, GC, C2, Cs, Lno+1, Next);
 
-	["errormod_401", '=' , Module] ->
-	    C2 = C#sconf{errormod_401 = list_to_atom(Module)},
-	    fload(FD, server, GC, C2, Cs, Lno+1, Next);
+        ["errormod_401", '=' , Module] ->
+            C2 = C#sconf{errormod_401 = list_to_atom(Module)},
+            fload(FD, server, GC, C2, Cs, Lno+1, Next);
 
         ["arg_rewrite_mod", '=', Module] ->
             C2 = C#sconf{arg_rewrite_mod = list_to_atom(Module)},
@@ -923,6 +951,7 @@ fload(FD, server, GC, C, Cs, Lno, Chars) ->
                 false ->
                     {error, ?F("Expect true|false at line ~w", [Lno])}
             end;
+
         ['<', "/server", '>'] ->
             fload(FD, globals, GC, undefined, [C|Cs], Lno+1, Next);
 
@@ -945,6 +974,8 @@ fload(FD, server, GC, C, Cs, Lno, Chars) ->
             fload(FD, server, GC, C2, Cs, Lno+1, Next);
 
         ["allowed_scripts", '=' | Suffixes] ->
+            io:format("Suf ~p~n", [Suffixes]),
+
             C2 = C#sconf{allowed_scripts = 
                          lists:map(fun(X)->element(1,mime_types:t(X)) end,
                                    Suffixes)},
@@ -967,8 +998,17 @@ fload(FD, server, GC, C, Cs, Lno, Chars) ->
                     {error, "Can't revproxy to an URL with a path "}
             end;
 
-	['<', "extra_cgi_vars", "dir", '=', Dir, '>'] ->
-	    fload(FD, extra_cgi_vars, GC, C, Cs, Lno+1, Next, {Dir, []});
+        ["fwdproxy", '=', Bool] ->
+            case is_bool(Bool) of
+                {true, Val} ->
+                    C2 = ?sc_set_forward_proxy(C, Val),
+                    fload(FD, server, GC, C2, Cs, Lno+1, Next);
+                false ->
+                    {error, ?F("Expect true|false at line ~w", [Lno])}
+            end;
+
+        ['<', "extra_cgi_vars", "dir", '=', Dir, '>'] ->
+            fload(FD, extra_cgi_vars, GC, C, Cs, Lno+1, Next, {Dir, []});
 
         ["statistics", '=', Bool] ->
             case is_bool(Bool) of
@@ -978,16 +1018,46 @@ fload(FD, server, GC, C, Cs, Lno, Chars) ->
                 false ->
                     {error, ?F("Expect true|false at line ~w", [Lno])}
             end;
+        ["fcgi_app_server", '=', Val] ->
+            case string_to_host_and_port(Val) of
+                {ok, Host, Port} ->
+                    C2 = C#sconf{fcgi_app_server_host = Host, fcgi_app_server_port = Port},
+                    fload(FD, server, GC, C2, Cs, Lno+1, Next);
+                {error, Reason} ->
+                    {error, ?F("Invalid fcgi_app_server ~p at line ~w: ~s", [Val, Lno, Reason])}
+            end;
+
+                
+
+        ["fcgi_trace_protocol", '=', Bool] ->
+            case is_bool(Bool) of
+                {true, Val} ->
+                    C2 = ?sc_set_fcgi_trace_protocol(C, Val),
+                    fload(FD, server, GC, C2, Cs, Lno+1, Next);
+                false ->
+                    {error, ?F("Expect true|false at line ~w", [Lno])}
+            end;
+
+        ["fcgi_log_app_error", '=', Bool] ->
+            case is_bool(Bool) of
+                {true, Val} ->
+                    C2 = ?sc_set_fcgi_log_app_error(C, Val),
+                    fload(FD, server, GC, C2, Cs, Lno+1, Next);
+                false ->
+                    {error, ?F("Expect true|false at line ~w", [Lno])}
+            end;
 
         [H|T] ->
-            {error, ?F("Unexpected input ~p at line ~w", [[H|T], Lno])}
+            {error, ?F("Unexpected input ~p at line ~w", [[H|T], Lno])};
+        Err ->
+            Err
     end;
 
 
 
 fload(FD, ssl, GC, C, Cs, Lno, Chars) ->
     Next = io:get_line(FD, ''),
-    case toks(Chars) of
+    case toks(Lno, Chars) of
         [] ->
             fload(FD, ssl, GC, C, Cs, Lno+1, Next);
 
@@ -1040,7 +1110,7 @@ fload(FD, ssl, GC, C, Cs, Lno, Chars) ->
             end;
         ["depth", '=', Val0] ->
             Val = (catch list_to_integer(Val0)),
-            case lists:member(Val, [1,2,3,4,5,6,7]) of
+            case lists:member(Val, [0, 1,2,3,4,5,6,7]) of
                 true when  is_record(C#sconf.ssl, ssl) ->
                     C2 = C#sconf{ssl = (C#sconf.ssl)#ssl{depth = Val}},
                     fload(FD, ssl, GC, C2, Cs, Lno+1, Next);
@@ -1048,7 +1118,7 @@ fload(FD, ssl, GC, C, Cs, Lno, Chars) ->
                     {error, ?F("Need to set option ssl to true before line ~w",
                                [Lno])};
                 _ ->
-                    {error, ?F("Expect reasonable integer at line ~w", [Lno])}
+                    {error, ?F("Expect integer 0..7 at line ~w", [Lno])}
             end;
         ["password", '=', Val] ->
             if 
@@ -1072,7 +1142,9 @@ fload(FD, ssl, GC, C, Cs, Lno, Chars) ->
             fload(FD, server, GC, C, Cs, Lno+1, Next);
 
         [H|T] ->
-            {error, ?F("Unexpected input ~p at line ~w", [[H|T], Lno])}
+            {error, ?F("Unexpected input ~p at line ~w", [[H|T], Lno])};
+        Err -> 
+            Err
     end;
 
 fload(FD, rss, _GC, _C, _Cs, Lno, eof) ->
@@ -1082,7 +1154,7 @@ fload(FD, rss, _GC, _C, _Cs, Lno, eof) ->
 fload(FD, rss, GC, C, Cs, Lno, Chars) ->
     %%?Debug("Chars: ~s", [Chars]),
     Next = io:get_line(FD, ''),
-    case toks(Chars) of
+    case toks(Lno, Chars) of
         [] ->
             fload(FD, rss, GC, C, Cs, Lno+1, Next);
         ['<', "/rss", '>'] ->
@@ -1112,7 +1184,9 @@ fload(FD, rss, GC, C, Cs, Lno, Chars) ->
             put(rss, [{rm_max, Value} | get(rss)]),
             fload(FD, rss, GC, C, Cs, Lno+1, Next);
         [H|T] ->
-            {error, ?F("Unexpected input ~p at line ~w", [[H|T], Lno])}
+            {error, ?F("Unexpected input ~p at line ~w", [[H|T], Lno])};
+        Err ->
+            Err
     end;
 
 fload(FD, opaque, _GC, _C, _Cs, Lno, eof) ->
@@ -1122,7 +1196,7 @@ fload(FD, opaque, _GC, _C, _Cs, Lno, eof) ->
 fload(FD, opaque, GC, C, Cs, Lno, Chars) ->
     %%?Debug("Chars: ~s", [Chars]),
     Next = io:get_line(FD, ''),
-    case toks(Chars) of
+    case toks(Lno, Chars) of
         [] ->
             fload(FD, opaque, GC, C, Cs, Lno+1, Next);
         ['<', "/opaque", '>'] ->
@@ -1141,7 +1215,9 @@ fload(FD, opaque, GC, C, Cs, Lno, Chars) ->
             C2 = C#sconf{opaque = [{Key, String_value} | C#sconf.opaque]},
             fload(FD, opaque, GC, C2, Cs, Lno+1, Next);
         [H|T] ->
-            {error, ?F("Unexpected input ~p at line ~w", [[H|T], Lno])}
+            {error, ?F("Unexpected input ~p at line ~w", [[H|T], Lno])};
+        Err ->
+            Err
     end.
 
 
@@ -1152,7 +1228,7 @@ fload(FD, server_auth, _GC, _C, _Cs, Lno, eof, _Auth) ->
 fload(FD, server_auth, GC, C, Cs, Lno, Chars, Auth) ->
     %%?Debug("Chars: ~s", [Chars]),
     Next = io:get_line(FD, ''),
-    case toks(Chars) of
+    case toks(Lno, Chars) of
         [] ->
             fload(FD, server_auth, GC, C, Cs, Lno+1, Next, Auth);
         ["dir", '=', Authdir] ->
@@ -1170,9 +1246,9 @@ fload(FD, server_auth, GC, C, Cs, Lno, Chars, Auth) ->
             A2 = Auth#auth{realm = Realm},
             fload(FD, server_auth, GC, C, Cs, Lno+1, Next, A2);
         ["authmod", '=', Mod] ->
-	    Mod2 = list_to_atom(Mod),
-	    code:ensure_loaded(Mod2),
-	    %% Add the auth header for the mod
+            Mod2 = list_to_atom(Mod),
+            code:ensure_loaded(Mod2),
+            %% Add the auth header for the mod
 	    H = try 
                     Mod2:get_header() ++ Auth#auth.headers
                 catch _:_ ->
@@ -1194,20 +1270,23 @@ fload(FD, server_auth, GC, C, Cs, Lno, Chars, Auth) ->
             A2 = Auth#auth{pam=Serv},
             fload(FD, server_auth, GC, C, Cs, Lno+1, Next, A2);
         ['<', "/auth", '>'] ->
-	    Pam = Auth#auth.pam,
-	    Users = Auth#auth.users,
-	    Realm = Auth#auth.realm,
-	    A2 =  case {Pam, Users} of
-		    {false, []} ->
-			  Auth;
-		      _ ->
-			  H = Auth#auth.headers ++ yaws:make_www_authenticate_header({realm, Realm}),
-			  Auth#auth{headers = H}
-		  end,
-	    C2 = C#sconf{authdirs = [A2|C#sconf.authdirs]},
+            Pam = Auth#auth.pam,
+            Users = Auth#auth.users,
+            Realm = Auth#auth.realm,
+            A2 =  case {Pam, Users} of
+                    {false, []} ->
+                          Auth;
+                      _ ->
+                          H = Auth#auth.headers ++ 
+                              yaws:make_www_authenticate_header({realm, Realm}),
+                          Auth#auth{headers = H}
+                  end,
+            C2 = C#sconf{authdirs = [A2|C#sconf.authdirs]},
             fload(FD, server, GC, C2, Cs, Lno+1, Next);
         [H|T] ->
-            {error, ?F("Unexpected input ~p at line ~w", [[H|T], Lno])}
+            {error, ?F("Unexpected input ~p at line ~w", [[H|T], Lno])};
+        Err ->
+            Err
     end;
 
 fload(FD, server_redirect, _GC, _C, _Cs, Lno, eof, _RedirMap) ->
@@ -1217,12 +1296,11 @@ fload(FD, server_redirect, _GC, _C, _Cs, Lno, eof, _RedirMap) ->
 fload(FD, server_redirect, GC, C, Cs, Lno, Chars, RedirMap) ->
     %%?Debug("Chars: ~s", [Chars]),
     Next = io:get_line(FD, ''),
-    Toks = toks(Chars),
+    Toks = toks(Lno, Chars),
     case Toks of
         [] ->
             fload(FD, server_redirect, GC, C, Cs, Lno+1, Next, RedirMap);
         [Path, '=', URL] ->
-            io:format(" = ~p ~p~n", [URL, Toks]),
             try yaws_api:parse_url(URL, sloppy) of
                 U when is_record(U, url) ->
                      fload(FD, server_redirect, GC, C, Cs, Lno+1, Next,
@@ -1231,7 +1309,6 @@ fload(FD, server_redirect, GC, C, Cs, Lno, Chars, RedirMap) ->
                     {error, ?F("bad redir ~p at line ~w", [URL, Lno])}
             end;
         [Path, '=', '=', URL] ->
-            io:format(" == ~p~n", [URL]),
             try yaws_api:parse_url(URL, sloppy) of
                 U when is_record(U, url) ->
                      fload(FD, server_redirect, GC, C, Cs, Lno+1, Next,
@@ -1243,7 +1320,9 @@ fload(FD, server_redirect, GC, C, Cs, Lno, Chars, RedirMap) ->
             C2 = C#sconf{redirect_map = lists:reverse(RedirMap)},
             fload(FD, server, GC, C2, Cs, Lno+1, Next);
         [H|T] ->
-            {error, ?F("Unexpected input ~p at line ~w", [[H|T], Lno])}
+            {error, ?F("Unexpected input ~p at line ~w", [[H|T], Lno])};
+        Err ->
+            Err
     end;
 
 fload(FD, extra_cgi_vars, _GC, _C, _Cs, Lno, eof, _EVars) ->
@@ -1252,7 +1331,7 @@ fload(FD, extra_cgi_vars, _GC, _C, _Cs, Lno, eof, _EVars) ->
 
 fload(FD, extra_cgi_vars, GC, C, Cs, Lno, Chars, EVars = {Dir, Vars}) ->
     Next = io:get_line(FD, ''),
-    case toks(Chars) of
+    case toks(Lno, Chars) of
         [] ->
             fload(FD, extra_cgi_vars, GC, C, Cs, Lno+1, Next, EVars);
         [Var, '=', Val] ->
@@ -1261,7 +1340,9 @@ fload(FD, extra_cgi_vars, GC, C, Cs, Lno, Chars, EVars = {Dir, Vars}) ->
             C2 = C#sconf{extra_cgi_vars = [EVars | C#sconf.extra_cgi_vars]},
             fload(FD, server, GC, C2, Cs, Lno+1, Next);
         [H|T] ->
-            {error, ?F("Unexpected input ~p at line ~w", [[H|T], Lno])}
+            {error, ?F("Unexpected input ~p at line ~w", [[H|T], Lno])};
+        Err ->
+            Err
     end.
 
 
@@ -1275,12 +1356,12 @@ is_bool(_) ->
 
 warn_dir(Type, Dir) ->
     case is_dir(Dir) of
-	true ->
-	    true;
-	false ->
-	    error_logger:format("Config Warning: Directory ~s for ~s doesn't exist~n",
-				[Dir, Type]),
-	    false
+        true ->
+            true;
+        false ->
+            error_logger:format("Config Warning: Directory ~s for ~s doesn't exist~n",
+                                [Dir, Type]),
+            false
     end.
 
 is_dir(Val) ->
@@ -1302,69 +1383,73 @@ is_file(Val) ->
 
 
 %% tokenizer
-toks(Chars) ->
-    toks(Chars, free, [], []). % two accumulators
+toks(Lno, Chars) ->
+    toks(Lno, Chars, free, [], []). % two accumulators
 
-toks([$#|_T], Mode, Ack, Tack) ->
-    toks([], Mode, Ack, Tack);
+toks(Lno, [$#|_T], Mode, Ack, Tack) ->
+    toks(Lno, [], Mode, Ack, Tack);
 
-toks([H|T], free, Ack, Tack) -> 
-                                                %?Debug("Char=~p", [H]),
-    case {is_quote(H), is_string_char(H), is_special(H), yaws:is_space(H)} of
+toks(Lno, [H|T], free, Ack, Tack) -> 
+    %%?Debug("Char=~p", [H]),
+    case {is_quote(H), is_string_char([H|T]),is_special(H), yaws:is_space(H)} of
         {_,_, _, true} ->
-            toks(T, free, Ack, Tack);
+            toks(Lno, T, free, Ack, Tack);
         {_,_, true, _} ->
-            toks(T, free, [], [list_to_atom([H]) | Tack]);
+            toks(Lno, T, free, [], [list_to_atom([H]) | Tack]);
         {_,true, _,_} ->
-            toks(T, string, [H], Tack);
+            toks(Lno, T, string, [H], Tack);
+        {_,utf8, _,_} ->
+            toks(Lno, tl(T), string, [H, hd(T)], Tack);
         {true,_, _,_} ->
-            toks(T, quote, [], Tack);
+            toks(Lno, T, quote, [], Tack);
         {false, false, false, false} ->
-            %% weird char, let's ignore it
-            toks(T, free, Ack, Tack)
+            {error, ?F("Unexpected character  <~p / ~c> at line ~w", 
+                       [H,H, Lno])}
     end;
-toks([C|T], string, Ack, Tack) -> 
+toks(Lno, [C|T], string, Ack, Tack) -> 
                                                 %?Debug("Char=~p", [C]),
-    case {is_backquote(C), is_quote(C), is_string_char(C), is_special(C), 
+    case {is_backquote(C), is_quote(C), is_string_char([C|T]), is_special(C), 
           yaws:is_space(C)} of
         {true, _, _, _,_} ->
-            toks(T, [backquote,string], Ack, Tack);
+            toks(Lno, T, [backquote,string], Ack, Tack);
         {_, _, true, _,_} ->
-            toks(T, string, [C|Ack], Tack);
+            toks(Lno, T, string, [C|Ack], Tack);
+        {_, _, utf8, _,_} ->
+            toks(Lno, tl(T), string, [C, hd(T)|Ack], Tack);
         {_, _, _, true, _} ->
-            toks(T, free, [], [list_to_atom([C]), lists:reverse(Ack)|Tack]);
+            toks(Lno, T, free, [], [list_to_atom([C]),lists:reverse(Ack)|Tack]);
         {_, true, _, _, _} ->
-            toks(T, quote, [], [lists:reverse(Ack)|Tack]);
+            toks(Lno, T, quote, [], [lists:reverse(Ack)|Tack]);
         {_, _, _, _, true} ->
-            toks(T, free, [], [lists:reverse(Ack)|Tack]);
+            toks(Lno, T, free, [], [lists:reverse(Ack)|Tack]);
         {false, false, false, false, false} ->
-            %% weird char, let's ignore it
-            toks(T, free, Ack, Tack)
+            {error, ?F("Unexpected character  <~p / ~c> at line ~w", 
+                       [C, C, Lno])}
     end;
-toks([C|T], quote, Ack, Tack) -> 
+toks(Lno, [C|T], quote, Ack, Tack) -> 
                                                 %?Debug("Char=~p", [C]),
     case {is_quote(C), is_backquote(C)} of
         {true, _} ->
-            toks(T, free, [], [lists:reverse(Ack)|Tack]);
+            toks(Lno, T, free, [], [lists:reverse(Ack)|Tack]);
         {_, true} ->
-            toks(T, [backquote,quote], [C|Ack], Tack);
+            toks(Lno, T, [backquote,quote], [C|Ack], Tack);
         {false, false} ->
-            toks(T, quote, [C|Ack], Tack)
+            toks(Lno, T, quote, [C|Ack], Tack)
     end;
-toks([C|T], [backquote,Mode], Ack, Tack) ->
-    toks(T, Mode, [C|Ack], Tack);
-toks([], string, Ack, Tack) ->
+toks(Lno, [C|T], [backquote,Mode], Ack, Tack) ->
+    toks(Lno, T, Mode, [C|Ack], Tack);
+toks(_Lno, [], string, Ack, Tack) ->
     lists:reverse([lists:reverse(Ack) | Tack]);
-toks([], free, _,Tack) ->
+toks(_Lno, [], free, _,Tack) ->
     lists:reverse(Tack).
 
-is_quote($") -> true ; 
+is_quote(34) -> true ;  %% $" but emacs mode can't handle it 
 is_quote(_)  -> false.
 
 is_backquote($\\) -> true ; 
 is_backquote(_)  -> false.
 
-is_string_char(C) ->
+is_string_char([C|T]) ->
     if
         $a =< C, C =< $z ->
             true;
@@ -1372,6 +1457,9 @@ is_string_char(C) ->
             true;
         $0 =< C, C =< $9 ->
             true;
+        C == 195 , T /= [] ->
+            %% FIXME check that [C, hd(T)] really is a char ?? how
+            utf8;
         true ->
             lists:member(C, [$., $/, $:, $_, $-, $~])
     end.
@@ -1380,11 +1468,51 @@ is_special(C) ->
     lists:member(C, [$=, $<, $>, $,]).
 
 
+parse_soap_srv_mods(['<', Module, ',' , Handler, ',', WsdlFile, '>' | Tail], 
+                    Ack) ->
+    case is_file(WsdlFile) of
+        true ->
+            S = { {list_to_atom(Module), list_to_atom(Handler)}, WsdlFile},
+            parse_soap_srv_mods(Tail, [S |Ack]);
+        false ->
+            {error, ?F("Bad wsdl file ~p", [WsdlFile])}
+    end;
+
+parse_soap_srv_mods(['<', Module, ',' , Handler, ',', WsdlFile, ',', 
+                     Prefix, '>' | Tail], Ack) ->
+    case is_file(WsdlFile) of
+        true ->
+            S = { {list_to_atom(Module), list_to_atom(Handler)}, 
+                  WsdlFile, Prefix},
+            parse_soap_srv_mods(Tail, [S |Ack]);
+        false ->
+            {error, ?F("Bad wsdl file ~p", [WsdlFile])}
+    end;
+
+parse_soap_srv_mods([ SoapSrvMod | _Tail], _Ack) ->
+    {error, ?F("Bad soap_srv_mods syntax: ~p", [SoapSrvMod])};
+
+parse_soap_srv_mods([], Ack) ->
+    {ok, Ack}.
 
 parse_appmods(['<', PathElem, ',' , AppMod, '>' | Tail], Ack) ->
     S = {PathElem , list_to_atom(AppMod)},
     parse_appmods(Tail, [S |Ack]);
 
+parse_appmods(['<', PathElem, ',' , AppMod, "exclude_paths" |Tail], Ack)->
+    Paths = lists:takewhile(fun(X) -> X /= '>' end, 
+                            Tail),
+    Tail2 = lists:dropwhile(fun(X) -> X /= '>' end, 
+                            Tail),
+    Tail3 = tl(Tail2),
+    
+    S = {PathElem , list_to_atom(AppMod), lists:map(
+                                            fun(Str) ->
+                                                    string:tokens(Str, "/")
+                                            end, Paths)},
+    parse_appmods(Tail3, [S |Ack]);
+
+
 parse_appmods([AppMod | Tail], Ack) ->
     %% just some simpleminded test to catch syntax errors in the config
     case AppMod of
@@ -1531,7 +1659,9 @@ eq_sconfs(S1,S2) ->
      S1#sconf.start_mod == S2#sconf.start_mod andalso
      S1#sconf.allowed_scripts == S2#sconf.allowed_scripts andalso
      S1#sconf.revproxy == S2#sconf.revproxy andalso
-     S1#sconf.soptions == S2#sconf.soptions).
+     S1#sconf.soptions == S2#sconf.soptions andalso
+     S1#sconf.fcgi_app_server_host == S2#sconf.fcgi_app_server_host andalso
+     S1#sconf.fcgi_app_server_port == S2#sconf.fcgi_app_server_port).
 
 
 
@@ -1679,15 +1809,18 @@ find_sc(_SC,[]) ->
 verify_upgrade_args(GC, Groups0) when is_record(GC, gconf) ->
     case is_groups(Groups0) of
         true ->
-            %% embeded code may give appmods as a list of strings
-            %% So the above is for backwards compatibility
-            %% appmods should be {StringPathElem, ModAtom} tuples
+            %% Embedded code may give appmods as a list of strings, or
+            %% appmods can be {StringPathElem,ModAtom} or
+            %% {StringPathElem,ModAtom,ExcludePathsList} tuples. Handle
+            %% all possible variants here.
             Groups = yaws:deepmap(
                        fun(SC) ->
                                SC#sconf{appmods =
                                         lists:map(
                                           fun({PE, Mod}) ->
                                                   {PE, Mod};
+                                             ({PE,Mod,Ex}) ->
+                                                  {PE,Mod,Ex};
                                              (AM) when is_list(AM) ->
                                                   {AM,list_to_atom(AM)};
                                              (AM) when is_atom(AM) ->
@@ -1732,4 +1865,3 @@ delete_sconf(SC) ->
 
 update_gconf(GC) ->
     ok = gen_server:call(yaws_server, {update_gconf, GC}, infinity).
-
