@@ -98,6 +98,7 @@ actl_trace(What) ->
 %%          {stop, Reason}
 %%----------------------------------------------------------------------
 init([]) ->
+    process_flag(trap_exit, true),
     {ok, #state{running = false, now = fmtnow()}}.
 
 %%----------------------------------------------------------------------
@@ -403,7 +404,9 @@ handle_info(minute10, State) ->
             ok
     end,
     {noreply, State#state{alogs= L,
-                          auth_log = wrap(State#state.auth_log, State)}}.
+                          auth_log = wrap(State#state.auth_log, State)}};
+handle_info({'EXIT', _, _}, State) ->
+    {noreply, State}.
 
 
 
@@ -455,6 +458,7 @@ wrap(AL, State) ->
 %% Returns: any (ignored by gen_server)
 %%----------------------------------------------------------------------
 terminate(_Reason, _State) ->
+    gen_event:delete_handler(error_logger, yaws_log_file_h, normal),
     ok.
 
 %%%----------------------------------------------------------------------
@@ -463,10 +467,21 @@ terminate(_Reason, _State) ->
 
 
 fmt_alog(Time, Ip, User, Req, Status,  Length, Referrer, UserAgent) ->
-    [fmt_ip(Ip), " - ", User, [$\s], Time, [$\s, $"], Req, [$",$\s], 
+    [fmt_ip(Ip), " - ", User, [$\s], Time, [$\s, $"], no_ctl(Req), [$",$\s], 
      Status, [$\s], Length, [$\s,$"], Referrer, [$",$\s,$"], UserAgent, [$",$\n]].
 
 
+%% Odd security advisory that only affects webservers where users are 
+%% somehow allowed to upload files that later can be downloaded.
+
+no_ctl([H|T]) when H < 32 ->
+    no_ctl(T);
+no_ctl([H|T]) ->
+    [H|no_ctl(T)];
+no_ctl([]) ->
+    [].
+
+
 fmt_ip(IP) when is_tuple(IP) ->
     inet_parse:ntoa(IP);
 fmt_ip(undefined) ->
@@ -476,23 +491,25 @@ fmt_ip(HostName) ->
 
 
 fmtnow() ->
-    {{Year, Month, Day}, {Hour, Min, Sec}} = calendar:local_time(),
-    [fill_zero(Day,2),"/",yaws:month(Month),"/",integer_to_list(Year),":",
-     fill_zero(Hour,2),":",fill_zero(Min,2),":",fill_zero(Sec,2)," ",zone()].
+    {{Year, Month, Day}, {Hour, Min, Sec}} = 
+        calendar:now_to_local_time(now()),
+    ["[",fill_zero(Day,2),"/",yaws:month(Month),"/",integer_to_list(Year),":",
+     fill_zero(Hour,2),":",fill_zero(Min,2),":",fill_zero(Sec,2)," ",zone(),"]"].
+    
 
 zone() ->
     Time = erlang:universaltime(),
     LocalTime = calendar:universal_time_to_local_time(Time),
     DiffSecs = calendar:datetime_to_gregorian_seconds(LocalTime) - 
         calendar:datetime_to_gregorian_seconds(Time),
-    zone((DiffSecs/3600)*100).
+    zone(DiffSecs div 3600, (DiffSecs rem 3600) div 60).
 
-%% Ugly reformatting code to get times like +0000 and -1300
 
-zone(Val) when Val < 0 ->
-    [$-|fill_zero(trunc(abs(Val)), 4)];
-zone(Val) when Val >= 0 ->
-    [$+|fill_zero(trunc(Val), 4)].
+
+zone(Hr, Min) when Hr < 0; Min < 0 ->
+    [$-, fill_zero(abs(Hr), 2), fill_zero(abs(Min), 2)];
+zone(Hr, Min) when Hr >= 0, Min >= 0 ->
+    [$+, fill_zero(abs(Hr), 2), fill_zero(abs(Min), 2)].
 
 fill_zero(N, Width) ->
     left_fill(N, Width, $0).
