@@ -7,7 +7,8 @@
 %% Way to invoke just one test
 start([F]) ->
     ?line {ok, _} = ibrowse:start_link(),
-    apply(app_test, F, []).
+    apply(app_test, F, []),
+    ibrowse:stop().
 
 start() ->
     ?line ok,
@@ -16,19 +17,22 @@ start() ->
     test1(),
     test2(),
     test3(),
-    sendfile_get().
+    appmod_test(),
+    streamcontent_test(),
+    sendfile_get(),
+    ibrowse:stop().
 
 
 test1() ->
     io:format("test1\n",[]),
-    L = lists:seq(1, 500),
+    L = lists:seq(1, 100),
     SELF = self(),
-    Pids = lists:map(fun(I) -> 
+    Pids = lists:map(fun(I) ->
 			     spawn(fun() -> slow_client(I, SELF) end)
 		     end, L),
     ?line ok = allow_connects(Pids, 5),
     ?line ok = collect_pids(Pids).
-    
+
 
 collect_pids([]) ->
     ok;
@@ -42,14 +46,14 @@ collect_pids(Pids) ->
 
 %% max 5 connectors at a time
 allow_connects([], _) ->
-    io:format("All pids connected \n",[]);
+    io:format("  test1: all pids connected \n",[]);
 allow_connects(Pids, 0) ->
     receive
 	{Pid, connected} ->
 	    allow_connects(lists:delete(Pid, Pids), 1)
     end;
 allow_connects(Pids, I) ->
-    receive 
+    receive
 	{Pid, allow} ->
 	    Pid ! allow,
 	    allow_connects(Pids, I-1);
@@ -61,15 +65,15 @@ allow_connects(Pids, I) ->
 
 slow_client(I, Top) ->
     Top ! {self(), allow},
-    receive 
+    receive
 	allow ->
 	    ok
     end,
-    ?line {ok, C} = gen_tcp:connect(localhost, 8000, [{active, false}, 
+    ?line {ok, C} = gen_tcp:connect(localhost, 8000, [{active, false},
 						      {packet, http}]),
     Top ! {self(), connected},
-    ?line ok = gen_tcp:send(C, "GET /1000.txt HTTP/1.1\n"
-			    "Host: localhost:8000\n\n"),
+    ?line ok = gen_tcp:send(C, "GET /1000.txt HTTP/1.1\r\n"
+			    "Host: localhost:8000\r\n\r\n"),
     ?line {ok, Sz} = get_cont_len(C),
     ?line ok = inet:setopts(C, [binary,{packet, 0}]),
     read_loop(C, I, Sz),
@@ -86,7 +90,7 @@ read_loop(C, I, Sz)  ->
 
 
 get_cont_len(C) ->
-    ?line {value, {http_header, _,_,_, LenStr}} = 
+    ?line {value, {http_header, _,_,_, LenStr}} =
 	lists:keysearch('Content-Length', 3, tftest:get_headers(C)),
     {ok, list_to_integer(LenStr)}.
 
@@ -95,9 +99,7 @@ get_cont_len(C) ->
 
 test2() ->
     io:format("test2\n",[]),
-    blkget(10),
-    timer:sleep(10000),
-    test3().
+    blkget(10).
 
 
 blkget(0) ->
@@ -105,8 +107,8 @@ blkget(0) ->
 blkget(I) ->
     spawn(fun() ->
                   {ok, C} = gen_tcp:connect(localhost, 8000, [{active, false}]),
-                  gen_tcp:send(C, "GET /2000.txt\n"
-                                  "Host: localhost:8000\n\n"),
+                  gen_tcp:send(C, "GET /2000.txt HTTP/1.1\r\n"
+                                  "Host: localhost:8000\r\n\r\n"),
                   gen_tcp:recv(C, 200),
                   timer:sleep(5000),
                   exit(normal)
@@ -118,12 +120,14 @@ blkget(I) ->
 
 test3() ->
     io:format("test3\n",[]),
-    ?line {ok, "200", _Headers, []} = ibrowse:send_req("http://localhost:8000", [], head),
+    ?line {ok, "200", _Headers, []} = ibrowse:send_req("http://localhost:8000",
+                                                       [], head),
     ok.
 
 server_options_test() ->
     io:format("server_options_test\n",[]),
-    {ok, S} = gen_tcp:connect("localhost", 8000, [{packet, raw}, list, {active, false}]),
+    {ok, S} = gen_tcp:connect("localhost", 8000, [{packet, raw}, list,
+                                                  {active, false}]),
     ok = gen_tcp:send(S, "OPTIONS * HTTP/1.1\r\nHost: localhost\r\n\r\n"),
     inet:setopts(S, [{packet, http}]),
     ?line ok = server_options_recv(S),
@@ -141,24 +145,27 @@ server_options_recv(S, [{http_header,_,'Allow',_,"GET, HEAD, OPTIONS, PUT, POST,
     do_server_options_recv(S, Hdrs);
 server_options_recv(S, [{http_header,_,'Content-Length',_,"0"}|_]=Hdrs) ->
     do_server_options_recv(S, Hdrs);
-server_options_recv(_S, _Hdrs) ->
+server_options_recv(S, [{http_header,_,'Date',_,_}|_]=Hdrs) ->
+    do_server_options_recv(S, Hdrs);
+server_options_recv(_S, [Hdr|_Hdrs]) ->
+    io:format("unexpected server options HTTP header: ~p~n", [Hdr]),
     error.
 do_server_options_recv(S, Hdrs) ->
-    {ok, Hdr} = gen_tcp:recv(S, 0, 2000),
+    {ok, Hdr} = gen_tcp:recv(S, 0, 5000),
     server_options_recv(S, [Hdr|Hdrs]).
 
 -define(SENDFILE_GET_TIMEOUT, 120000).
 
 sendfile_get() ->
     io:format("sendfile_get\n",[]),
-    L = lists:seq(1, 20),
+    L = lists:seq(1, 5),
     SELF = self(),
     K1 = lists:map(
              fun(_) ->
                      spawn(fun() ->
-                                   ?line {ok, "200", _Headers, _} = 
+                                   ?line {ok, "200", _Headers, _} =
                                        ibrowse:send_req(
-                                         "http://localhost:8000/1000.txt", 
+                                         "http://localhost:8000/1000.txt",
                                          [], get, [], [], ?SENDFILE_GET_TIMEOUT),
                                    SELF ! {self(), k1, done}
                            end)
@@ -167,36 +174,36 @@ sendfile_get() ->
     K2 = lists:map(
              fun(_) ->
                      spawn(fun() ->
-                                   ?line {ok, "200", _Headers, _} = 
+                                   ?line {ok, "200", _Headers, _} =
                                        ibrowse:send_req(
-                                         "http://localhost:8000/2000.txt", 
+                                         "http://localhost:8000/2000.txt",
                                          [], get, [], [], ?SENDFILE_GET_TIMEOUT),
                                    SELF ! {self(), k2, done}
                            end)
              end, L),
 
-    %% K3 = lists:map(
-    %%          fun(_) ->
-    %%                  spawn(fun() ->
-    %%                                ?line {ok, "200", _Headers, _} = 
-    %%                                    ibrowse:send_req(
-    %%                                      "http://localhost:8000/3000.txt", 
-    %%                                      [], get, [], [], ?SENDFILE_GET_TIMEOUT),
-    %%                                SELF ! {self(), k3, done}
-    %%                        end)
-    %%          end, L),
+    K3 = lists:map(
+             fun(_) ->
+                     spawn(fun() ->
+                                   ?line {ok, "200", _Headers, _} =
+                                       ibrowse:send_req(
+                                         "http://localhost:8000/3000.txt",
+                                         [], get, [], [], ?SENDFILE_GET_TIMEOUT),
+                                   SELF ! {self(), k3, done}
+                           end)
+             end, L),
 
 
- %   io:format("K3 = ~p~n", [K3]),
     collect(K1, 1, k1),
     collect(K2, 1, k2),
-%    collect(K3, 1, k3),
+    collect(K3, 1, k3),
+    io:format("\n",[]),
     ok.
 
 collect([], _, _) ->
     ok;
 collect(L, Count, Tag) ->
-    receive 
+    receive
         {Pid, Tag, done} ->
             io:format("(~p ~p)", [Tag, Count]),
             collect(lists:delete(Pid, L), Count+1, Tag)
@@ -204,15 +211,36 @@ collect(L, Count, Tag) ->
             io:format("TIMEOUT ~p\n~p~n",[process_info(self()), L]),
             ?line exit(timeout)
     end.
-                    
-                
-                
-                              
-    
-           
-
-
 
+-define(APPMOD_HEADER, "Appmod-Called").
 
+appmod_test() ->
+    io:format("appmod_test\n",[]),
+    Uri1 = "http://localhost:8002/",
+    ?line {ok, "200", Headers1, _} = ibrowse:send_req(Uri1, [], get),
+    ?line "true" = proplists:get_value(?APPMOD_HEADER, Headers1),
+    Uri2 = "http://localhost:8003/",
+    ?line {ok, "200", Headers2, _} = ibrowse:send_req(Uri2, [], get),
+    ?line "true" = proplists:get_value(?APPMOD_HEADER, Headers2),
+    Uri3 = "http://localhost:8003/icons/layout.gif",
+    ?line {ok, "200", Headers3, _} = ibrowse:send_req(Uri3, [], get),
+    ?line false = proplists:get_value(?APPMOD_HEADER, Headers3, false),
+    Uri4 = "http://localhost:8004/non_root_appmod",
+    ?line {ok, "200", Headers4, _} = ibrowse:send_req(Uri4, [], get),
+    ?line "true" = proplists:get_value(?APPMOD_HEADER, Headers4),
+    ok.
 
+streamcontent_test() ->
+    io:format("streamcontent_test\n",[]),
+    Uri = "http://localhost:8000/streamtest",
+    ?line {ok, "200", Headers, Body} = ibrowse:send_req(Uri, [], get),
+    ?line "chunked" = proplists:get_value("Transfer-Encoding", Headers),
+    ?line Body = "this is an iolist",
+    ok.
 
+%% used for appmod tests
+%%
+out(A) ->
+    %% add our special header to mark that we were here
+    [{status, 200},
+     {header, {?APPMOD_HEADER, "true"}}].
